@@ -27,20 +27,21 @@ __all__ = ['maximum_flow',
            'minimum_cut_value']
 
 
-def maximum_flow(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+def maximum_flow(flowG, _s, _t,
+                 capacity='capacity', flow_func=None, **kwargs):
     """Find a maximum single-commodity flow.
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -165,26 +166,27 @@ def maximum_flow(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     if not callable(flow_func):
         raise nx.NetworkXError("flow_func has to be callable.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=False, **kwargs)
-    flow_dict = build_flow_dict(G, R)
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=False, **kwargs)
+    flow_dict = build_flow_dict(flowG, R)
 
     return (R.graph['flow_value'], flow_dict)
 
 
-def maximum_flow_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+def maximum_flow_value(flowG, _s, _t,
+                       capacity='capacity', flow_func=None, **kwargs):
     """Find the value of maximum single-commodity flow.
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -303,12 +305,13 @@ def maximum_flow_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     if not callable(flow_func):
         raise nx.NetworkXError("flow_func has to be callable.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=True, **kwargs)
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=True, **kwargs)
 
     return R.graph['flow_value']
 
 
-def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+def minimum_cut(flowG, _s, _t,
+                capacity='capacity', flow_func=None, **kwargs):
     """Compute the value and the node partition of a minimum (s, t)-cut.
 
     Use the max-flow min-cut theorem, i.e., the capacity of a minimum
@@ -316,15 +319,15 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -454,8 +457,8 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     if kwargs.get('cutoff') is not None and flow_func in flow_funcs:
         raise nx.NetworkXError("cutoff should not be specified.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=True, **kwargs)
-    # Remove saturated edges from the residual network 
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=True, **kwargs)
+    # Remove saturated edges from the residual network
     cutset = [(u, v, d) for u, v, d in R.edges(data=True)
               if d['flow'] == d['capacity']]
     R.remove_edges_from(cutset)
@@ -463,8 +466,8 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     # Then, reachable and non reachable nodes from source in the
     # residual network form the node partition that defines
     # the minimum cut.
-    non_reachable = set(dict(nx.shortest_path_length(R, target=t)))
-    partition = (set(G) - non_reachable, non_reachable)
+    non_reachable = set(dict(nx.shortest_path_length(R, target=_t)))
+    partition = (set(flowG) - non_reachable, non_reachable)
     # Finaly add again cutset edges to the residual network to make
     # sure that it is reusable.
     if cutset is not None:
@@ -472,7 +475,8 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     return (R.graph['flow_value'], partition)
 
 
-def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+def minimum_cut_value(flowG, _s, _t,
+                      capacity='capacity', flow_func=None, **kwargs):
     """Compute the value of a minimum (s, t)-cut.
 
     Use the max-flow min-cut theorem, i.e., the capacity of a minimum
@@ -480,15 +484,15 @@ def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -604,6 +608,6 @@ def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     if kwargs.get('cutoff') is not None and flow_func in flow_funcs:
         raise nx.NetworkXError("cutoff should not be specified.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=True, **kwargs)
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=True, **kwargs)
 
     return R.graph['flow_value']
