@@ -1,22 +1,26 @@
-"""
-Graph products.
-"""
 #    Copyright (C) 2011 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-import networkx as nx
+#
+# Authors:
+#     Aric Hagberg <hagberg@lanl.gov>
+#     Pieter Swart <swart@lanl.gov>
+#     Dan Schult <dschult@colgate.edu>
+#     Ben Edwards <bedwards@cs.unm.edu>
+"""
+Graph products.
+"""
 from itertools import product
 
-__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
-                            'Pieter Swart (swart@lanl.gov)',
-                            'Dan Schult(dschult@colgate.edu)'
-                            'Ben Edwards(bedwards@cs.unm.edu)'])
+import networkx as nx
+from networkx.utils import not_implemented_for
 
 __all__ = ['tensor_product', 'cartesian_product',
-           'lexicographic_product', 'strong_product', 'power']
+           'lexicographic_product', 'strong_product', 'power',
+           'rooted_product']
 
 
 def _dict_product(d1, d2):
@@ -26,54 +30,54 @@ def _dict_product(d1, d2):
 # Generators for producting graph products
 def _node_product(G, H):
     for u, v in product(G, H):
-        yield ((u, v), _dict_product(G.node[u], H.node[v]))
+        yield ((u, v), _dict_product(G.nodes[u], H.nodes[v]))
 
 
 def _directed_edges_cross_edges(G, H):
     if not G.is_multigraph() and not H.is_multigraph():
-        for u, v, c in G.edges_iter(data=True):
-            for x, y, d in H.edges_iter(data=True):
+        for u, v, c in G.edges(data=True):
+            for x, y, d in H.edges(data=True):
                 yield (u, x), (v, y), _dict_product(c, d)
     if not G.is_multigraph() and H.is_multigraph():
-        for u, v, c in G.edges_iter(data=True):
-            for x, y, k, d in H.edges_iter(data=True, keys=True):
+        for u, v, c in G.edges(data=True):
+            for x, y, k, d in H.edges(data=True, keys=True):
                 yield (u, x), (v, y), k, _dict_product(c, d)
     if G.is_multigraph() and not H.is_multigraph():
-        for u, v, k, c in G.edges_iter(data=True, keys=True):
-            for x, y, d in H.edges_iter(data=True):
+        for u, v, k, c in G.edges(data=True, keys=True):
+            for x, y, d in H.edges(data=True):
                 yield (u, x), (v, y), k, _dict_product(c, d)
     if G.is_multigraph() and H.is_multigraph():
-        for u, v, j, c in G.edges_iter(data=True, keys=True):
-            for x, y, k, d in H.edges_iter(data=True, keys=True):
+        for u, v, j, c in G.edges(data=True, keys=True):
+            for x, y, k, d in H.edges(data=True, keys=True):
                 yield (u, x), (v, y), (j, k), _dict_product(c, d)
 
 
 def _undirected_edges_cross_edges(G, H):
     if not G.is_multigraph() and not H.is_multigraph():
-        for u, v, c in G.edges_iter(data=True):
-            for x, y, d in H.edges_iter(data=True):
+        for u, v, c in G.edges(data=True):
+            for x, y, d in H.edges(data=True):
                 yield (v, x), (u, y), _dict_product(c, d)
     if not G.is_multigraph() and H.is_multigraph():
-        for u, v, c in G.edges_iter(data=True):
-            for x, y, k, d in H.edges_iter(data=True, keys=True):
+        for u, v, c in G.edges(data=True):
+            for x, y, k, d in H.edges(data=True, keys=True):
                 yield (v, x), (u, y), k, _dict_product(c, d)
     if G.is_multigraph() and not H.is_multigraph():
-        for u, v, k, c in G.edges_iter(data=True, keys=True):
-            for x, y, d in H.edges_iter(data=True):
+        for u, v, k, c in G.edges(data=True, keys=True):
+            for x, y, d in H.edges(data=True):
                 yield (v, x), (u, y), k, _dict_product(c, d)
     if G.is_multigraph() and H.is_multigraph():
-        for u, v, j, c in G.edges_iter(data=True, keys=True):
-            for x, y, k, d in H.edges_iter(data=True, keys=True):
+        for u, v, j, c in G.edges(data=True, keys=True):
+            for x, y, k, d in H.edges(data=True, keys=True):
                 yield (v, x), (u, y), (j, k), _dict_product(c, d)
 
 
 def _edges_cross_nodes(G, H):
     if G.is_multigraph():
-        for u, v, k, d in G.edges_iter(data=True, keys=True):
+        for u, v, k, d in G.edges(data=True, keys=True):
             for x in H:
                 yield (u, x), (v, x), k, d
     else:
-        for u, v, d in G.edges_iter(data=True):
+        for u, v, d in G.edges(data=True):
             for x in H:
                 if H.is_multigraph():
                     yield (u, x), (v, x), None, d
@@ -84,11 +88,11 @@ def _edges_cross_nodes(G, H):
 def _nodes_cross_edges(G, H):
     if H.is_multigraph():
         for x in G:
-            for u, v, k, d in H.edges_iter(data=True, keys=True):
+            for u, v, k, d in H.edges(data=True, keys=True):
                 yield (x, u), (x, v), k, d
     else:
         for x in G:
-            for u, v, d in H.edges_iter(data=True):
+            for u, v, d in H.edges(data=True):
                 if G.is_multigraph():
                     yield (x, u), (x, v), None, d
                 else:
@@ -97,12 +101,12 @@ def _nodes_cross_edges(G, H):
 
 def _edges_cross_nodes_and_nodes(G, H):
     if G.is_multigraph():
-        for u, v, k, d in G.edges_iter(data=True, keys=True):
+        for u, v, k, d in G.edges(data=True, keys=True):
             for x in H:
                 for y in H:
                     yield (u, x), (v, y), k, d
     else:
-        for u, v, d in G.edges_iter(data=True):
+        for u, v, d in G.edges(data=True):
             for x in H:
                 for y in H:
                     if H.is_multigraph():
@@ -127,10 +131,10 @@ def _init_product_graph(G, H):
 def tensor_product(G, H):
     r"""Return the tensor product of G and H.
 
-    The tensor product P of the graphs G and H has a node set that
-    is the Cartesian product of the node sets, :math:`V(P)=V(G) \times V(H)`.
-    P has an edge ((u,v),(x,y)) if and only if (u,x) is an edge in G
-    and (v,y) is an edge in H.
+    The tensor product $P$ of the graphs $G$ and $H$ has a node set that
+    is the tensor product of the node sets, $V(P)=V(G) \times V(H)$.
+    $P$ has an edge $((u,v), (x,y))$ if and only if $(u,x)$ is an edge in $G$
+    and $(v,y)$ is an edge in $H$.
 
     Tensor product is sometimes also referred to as the categorical product,
     direct product, cardinal product or conjunction.
@@ -139,7 +143,7 @@ def tensor_product(G, H):
     Parameters
     ----------
     G, H: graphs
-     Networkx graphs. 
+     Networkx graphs.
 
     Returns
     -------
@@ -162,10 +166,10 @@ def tensor_product(G, H):
     --------
     >>> G = nx.Graph()
     >>> H = nx.Graph()
-    >>> G.add_node(0,a1=True)
-    >>> H.add_node('a',a2='Spam')
-    >>> P = nx.tensor_product(G,H)
-    >>> P.nodes()
+    >>> G.add_node(0, a1=True)
+    >>> H.add_node('a', a2='Spam')
+    >>> P = nx.tensor_product(G, H)
+    >>> list(P)
     [(0, 'a')]
 
     Edge attributes and edge keys (for multigraphs) are also copied to the
@@ -176,17 +180,17 @@ def tensor_product(G, H):
     GH.add_edges_from(_directed_edges_cross_edges(G, H))
     if not GH.is_directed():
         GH.add_edges_from(_undirected_edges_cross_edges(G, H))
-    GH.name = "Tensor product(" + G.name + "," + H.name + ")"
     return GH
 
 
 def cartesian_product(G, H):
-    """Return the Cartesian product of G and H.
+    r"""Return the Cartesian product of G and H.
 
-    The Cartesian product P of the graphs G and H has a node set that
-    is the Cartesian product of the node sets, :math:`V(P)=V(G) \times V(H)`.
-    P has an edge ((u,v),(x,y)) if and only if either u is equal to x and
-    v & y are adjacent in H or if v is equal to y and u & x are adjacent in G.
+    The Cartesian product $P$ of the graphs $G$ and $H$ has a node set that
+    is the Cartesian product of the node sets, $V(P)=V(G) \times V(H)$.
+    $P$ has an edge $((u,v),(x,y))$ if and only if either $u$ is equal to $x$
+    and both $v$ and $y$ are adjacent in $H$ or if $v$ is equal to $y$ and
+    both $u$ and $x$ are adjacent in $G$.
 
     Parameters
     ----------
@@ -214,10 +218,10 @@ def cartesian_product(G, H):
     --------
     >>> G = nx.Graph()
     >>> H = nx.Graph()
-    >>> G.add_node(0,a1=True)
-    >>> H.add_node('a',a2='Spam')
-    >>> P = nx.cartesian_product(G,H)
-    >>> P.nodes()
+    >>> G.add_node(0, a1=True)
+    >>> H.add_node('a', a2='Spam')
+    >>> P = nx.cartesian_product(G, H)
+    >>> list(P)
     [(0, 'a')]
 
     Edge attributes and edge keys (for multigraphs) are also copied to the
@@ -230,17 +234,16 @@ def cartesian_product(G, H):
     GH.add_nodes_from(_node_product(G, H))
     GH.add_edges_from(_edges_cross_nodes(G, H))
     GH.add_edges_from(_nodes_cross_edges(G, H))
-    GH.name = "Cartesian product(" + G.name + "," + H.name + ")"
     return GH
 
 
 def lexicographic_product(G, H):
-    """Return the lexicographic product of G and H.
+    r"""Return the lexicographic product of G and H.
 
-    The lexicographical product P of the graphs G and H has a node set that
-    is the Cartesian product of the node sets, $V(P)=V(G) \times V(H)$.
-    P has an edge ((u,v),(x,y)) if and only if (u,v) is an edge in G
-    or u==v and (x,y) is an edge in H.
+    The lexicographical product $P$ of the graphs $G$ and $H$ has a node set
+    that is the Cartesian product of the node sets, $V(P)=V(G) \times V(H)$.
+    $P$ has an edge $((u,v), (x,y))$ if and only if $(u,v)$ is an edge in $G$
+    or $u==v$ and $(x,y)$ is an edge in $H$.
 
     Parameters
     ----------
@@ -268,10 +271,10 @@ def lexicographic_product(G, H):
     --------
     >>> G = nx.Graph()
     >>> H = nx.Graph()
-    >>> G.add_node(0,a1=True)
-    >>> H.add_node('a',a2='Spam')
-    >>> P = nx.lexicographic_product(G,H)
-    >>> P.nodes()
+    >>> G.add_node(0, a1=True)
+    >>> H.add_node('a', a2='Spam')
+    >>> P = nx.lexicographic_product(G, H)
+    >>> list(P)
     [(0, 'a')]
 
     Edge attributes and edge keys (for multigraphs) are also copied to the
@@ -283,19 +286,18 @@ def lexicographic_product(G, H):
     GH.add_edges_from(_edges_cross_nodes_and_nodes(G, H))
     # For each x in G, only if there is an edge in H
     GH.add_edges_from(_nodes_cross_edges(G, H))
-    GH.name = "Lexicographic product(" + G.name + "," + H.name + ")"
     return GH
 
 
 def strong_product(G, H):
-    """Return the strong product of G and H.
+    r"""Return the strong product of G and H.
 
-    The strong product P of the graphs G and H has a node set that
+    The strong product $P$ of the graphs $G$ and $H$ has a node set that
     is the Cartesian product of the node sets, $V(P)=V(G) \times V(H)$.
-    P has an edge ((u,v),(x,y)) if and only if
-    u==v and (x,y) is an edge in H, or
-    x==y and (u,v) is an edge in G, or
-    (u,v) is an edge in G and (x,y) is an edge in H.
+    $P$ has an edge $((u,v), (x,y))$ if and only if
+    $u==v$ and $(x,y)$ is an edge in $H$, or
+    $x==y$ and $(u,v)$ is an edge in $G$, or
+    $(u,v)$ is an edge in $G$ and $(x,y)$ is an edge in $H$.
 
     Parameters
     ----------
@@ -323,10 +325,10 @@ def strong_product(G, H):
     --------
     >>> G = nx.Graph()
     >>> H = nx.Graph()
-    >>> G.add_node(0,a1=True)
-    >>> H.add_node('a',a2='Spam')
-    >>> P = nx.strong_product(G,H)
-    >>> P.nodes()
+    >>> G.add_node(0, a1=True)
+    >>> H.add_node('a', a2='Spam')
+    >>> P = nx.strong_product(G, H)
+    >>> list(P)
     [(0, 'a')]
 
     Edge attributes and edge keys (for multigraphs) are also copied to the
@@ -339,73 +341,77 @@ def strong_product(G, H):
     GH.add_edges_from(_directed_edges_cross_edges(G, H))
     if not GH.is_directed():
         GH.add_edges_from(_undirected_edges_cross_edges(G, H))
-    GH.name = "Strong product(" + G.name + "," + H.name + ")"
     return GH
 
 
+@not_implemented_for('directed')
+@not_implemented_for('multigraph')
 def power(G, k):
     """Returns the specified power of a graph.
 
-    The `k`-th power of a simple graph `G = (V, E)` is the graph
-    `G^k` whose vertex set is `V`,  two distinct vertices `u,v` are
-    adjacent in `G^k` if and only if the shortest path
-    distance between `u` and `v` in `G` is at most `k`.
-
+    The $k$th power of a simple graph $G$, denoted $G^k$, is a
+    graph on the same set of nodes in which two distinct nodes $u$ and
+    $v$ are adjacent in $G^k$ if and only if the shortest path
+    distance between $u$ and $v$ in $G$ is at most $k$.
 
     Parameters
     ----------
-    G: graph
-      A NetworkX simple graph object.
-    k: positive integer
-      The power to which to raise the graph `G`.
+    G : graph
+        A NetworkX simple graph object.
+
+    k : positive integer
+        The power to which to raise the graph `G`.
 
     Returns
     -------
     NetworkX simple graph
-      `G` to the `k`-th power.
+        `G` to the power `k`.
 
     Raises
     ------
-    :exc:`ValueError`
-      If the exponent `k` is not positive.
+    ValueError
+        If the exponent `k` is not positive.
 
-    NetworkXError
-      If G is not a simple graph.
+    NetworkXNotImplemented
+        If `G` is not a simple graph.
 
     Examples
     --------
+    The number of edges will never decrease when taking successive
+    powers:
+
     >>> G = nx.path_graph(4)
-    >>> nx.power(G,2).edges()
+    >>> list(nx.power(G, 2).edges)
     [(0, 1), (0, 2), (1, 2), (1, 3), (2, 3)]
-    >>> nx.power(G,3).edges()
+    >>> list(nx.power(G, 3).edges)
     [(0, 1), (0, 2), (0, 3), (1, 2), (1, 3), (2, 3)]
 
-    A complete graph of order n is returned if *k* is greater than equal to n/2
-    for a cycle graph of even order n, and if *k* is greater than equal to
-    (n-1)/2 for a cycle graph of odd order.
+    The `k`th power of a cycle graph on *n* nodes is the complete graph
+    on *n* nodes, if `k` is at least ``n // 2``:
 
     >>> G = nx.cycle_graph(5)
-    >>> nx.power(G,2).edges() == nx.complete_graph(5).edges()
+    >>> H = nx.complete_graph(5)
+    >>> nx.is_isomorphic(nx.power(G, 2), H)
     True
     >>> G = nx.cycle_graph(8)
-    >>> nx.power(G,4).edges() == nx.complete_graph(8).edges()
+    >>> H = nx.complete_graph(8)
+    >>> nx.is_isomorphic(nx.power(G, 4), H)
     True
 
     References
     ----------
-    .. [1] J. A. Bondy, U. S. R. Murty, Graph Theory. Springer, 2008.
+    .. [1] J. A. Bondy, U. S. R. Murty, *Graph Theory*. Springer, 2008.
 
     Notes
     -----
-    Exercise 3.1.6 of *Graph Theory* by J. A. Bondy and U. S. R. Murty [1]_.
-
+    This definition of "power graph" comes from Exercise 3.1.6 of
+    *Graph Theory* by Bondy and Murty [1]_.
 
     """
     if k <= 0:
         raise ValueError('k must be a positive integer')
-    if G.is_multigraph() or G.is_directed():
-        raise nx.NetworkXError("G should be a simple graph")
     H = nx.Graph()
+    H.add_nodes_from(G)
     # update BFS code to ignore self loops.
     for n in G:
         seen = {}                  # level (number of hops) when seen in BFS
@@ -420,8 +426,46 @@ def power(G, k):
                 if v not in seen:
                     seen[v] = level         # set the level of vertex v
                     nextlevel.update(G[v])  # add neighbors of v
-            if (k <= level):
+            if k <= level:
                 break
-            level = level + 1
+            level += 1
         H.add_edges_from((n, nbr) for nbr in seen)
     return H
+
+
+@not_implemented_for('multigraph')
+def rooted_product(G, H, root):
+    """ Return the rooted product of graphs G and H rooted at root in H.
+
+    A new graph is constructed representing the rooted product of
+    the inputted graphs, G and H, with a root in H.
+    A rooted product duplicates H for each nodes in G with the root
+    of H corresponding to the node in G. Nodes are renamed as the direct
+    product of G and H. The result is a subgraph of the cartesian product.
+
+    Parameters
+    ----------
+    G,H : graph
+       A NetworkX graph
+    root : node
+       A node in H
+
+    Returns
+    -------
+    R : The rooted product of G and H with a specified root in H
+
+    Notes
+    -----
+    The nodes of R are the Cartesian Product of the nodes of G and H.
+    The nodes of G and H are not relabeled.
+    """
+    if root not in H:
+        raise nx.NetworkXError('root must be a vertex in H')
+
+    R = nx.Graph()
+    R.add_nodes_from(product(G, H))
+
+    R.add_edges_from(((e[0], root), (e[1], root)) for e in G.edges())
+    R.add_edges_from(((g, e[0]), (g, e[1])) for g in G for e in H.edges())
+
+    return R
